import { 
  select, 
  csv, 
  json,
  scaleLinear, 
  scaleTime,
  scalePoint,
  min,
  max, 
  scaleBand, 
  axisLeft,
  axisBottom,
  format,
  extent,
  timeFormat
} from 'd3'

let svg = select('svg')

const height = +svg.attr('height')
const width = +svg.attr('width')

const render = data => {
  const margin = { top: 50, right: 40, bottom: 77, left: 100 }
  const innerWidth = width - margin.left - margin.right
  const innerHeight = height - margin.top - margin.bottom  
    
  const quantidade = d => d.quantidade
  const datahora = d => d.data
  const estoque = d => d.estoque  
    
	const escalaDataHora = scalePoint()
  	.domain(data.map(d => datahora(d)))
    .range([0, innerWidth])
		.padding(0.5)
	 	.align(-0.5)

  const escalaEstoque = scaleBand()
  	.domain(data.map(d => estoque(d)))
    .range([0, innerHeight])  	
  
  const tamanhoColuna = escalaDataHora.step() 
  const tamanhoLinha = escalaEstoque.bandwidth()
  
  const escalaQuantidade = scaleLinear()
    .domain([0, max(data, d => quantidade(d))])
    .range([ 0, tamanhoColuna * .9 ])
   
  svg = svg.append('g')
	.attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  const xAxisTickformat = date => timeFormat('%d/%m/%Y %H:%M:%S')(date)
  const xAxis = axisBottom(escalaDataHora)
  	.tickFormat(xAxisTickformat)
  	// .tickSize(-innerHeight)
  	.tickPadding(15)
  
  const xAxisG = svg.append('g').call(xAxis)
  	.attr('transform', `translate(0, ${innerHeight})`)
   .selectAll("text")
    .attr("y", 0)
    .attr("x", 9)
    .attr("dy", ".35em")
    .attr("transform", "rotate(45)")
    .style("text-anchor", "start")
  
  const yAxis = axisLeft(escalaEstoque)
  	.tickSizeInner(-innerWidth)  
  const yAxisG = svg.append('g')
    .call(yAxis)  
  	.attr('font-size', 15)   
  	.attr("class", "y axis")  
  		.selectAll(".y.axis .tick line")
  			.attr("transform", "translate(0," + escalaEstoque.bandwidth()/2 + ")")

  const groups = svg.selectAll('.quantidades')
  	.data(data)
  		.attr('class', 'quantidades')
  
  const groupsEnter = groups.enter().append('g')
    groupsEnter      
      .merge(groups)
        .attr('transform', (d) => 
             `translate(${(escalaDataHora(datahora(d)) + ((tamanhoColuna) - escalaQuantidade(quantidade(d))) / 2)}, ${(escalaEstoque(estoque(d)) + (tamanhoLinha / 4 ))})`)
  
    groupsEnter.append('rect')
    .merge(groups.select('rect'))
   	 .attr('width', d => escalaQuantidade(quantidade(d)))
   	 .attr('height', tamanhoLinha/2)
     .attr('rx', '2')
  
    groupsEnter.append('text')
  		.merge(groups.select('text'))
  			.text(d => d.quantidade)
        .attr('x', d => escalaQuantidade(quantidade(d)) / 2)
  			.attr('y', tamanhoLinha + -45)
  			.attr('class', 'texto-quantidade')
  
  svg.append('text')
  	.attr('class', 'title')
  	.attr('y', -10)
  	.attr('x', innerWidth / 2)
  	.attr('text-anchor', 'middle')
  	.text('Movimentação de Estoque')
}

json('data.json').then(data => {
  data.forEach(d=>{
  	d.data = new Date(d.data)
    d.quantidade = +d.quantidade
  })
  render(data)
})